@@ -1,9 +1,9 @@
 import re
 from copy import deepcopy
 from typing import Dict
-
+from libinsitu import STATION_NAME_VAR
 from libinsitu.common import parse_value, DATA_VARS, read_res, CDL_PATH, LONGITUDE_VAR, LATITUDE_VAR, ELEVATION_VAR, \
-    fillShortName, TIME_VAR
+    fill_str, TIME_VAR
 from libinsitu.log import info, warning
 
 
@@ -34,24 +34,29 @@ def replace_placeholders(strval, attributes) :
 
     return re.sub(r'{\w+}', repl, strval)
 
-def parse_cdl(lines, attributes) :
+def parse_cdl(lines, attributes=dict()) :
+    """ Parse CDL file """
 
     res = CDL()
 
     section = None
+    curr_var = None;
 
-    """ Parse CDL file """
     for line in lines :
         line = line.strip()
 
         # Skip comments
-        if line.startswith("//") or len(line) == 0:
+        if line.startswith("#") or len(line) == 0:
             continue
 
+        if "#" in line :
+            line = line.split("#")[0]
+            line = line.strip()
+
         # key = value
         if "=" in line :
             line = line.strip(";")
-            key, val = line.split("=")
+            key, val = line.split("=", 1)
             key = key.strip()
             val = parse_value(val.strip())
 
@@ -63,6 +68,9 @@ def parse_cdl(lines, attributes) :
 
             elif section == "variables" :
                 varname, attrname = key.split(":")
+
+                if varname == "*" :
+                    varname = curr_var
                 if isinstance(val, str) :
                     val = replace_placeholders(val, attributes)
                 if varname == "" :
@@ -89,6 +97,8 @@ def parse_cdl(lines, attributes) :
 
             if type == "char" :
                 type ="c"
+            elif type == "string" :
+                type = str
             elif type == "float" :
                 type="f4"
             elif type == "int" :
@@ -101,6 +111,7 @@ def parse_cdl(lines, attributes) :
                 var, dims = var.split("(")
                 dims = dims.strip(")").strip().split(",")
             res.variables[var] = Variable(var, type, dims)
+            curr_var = var
         else :
             raise Exception("Bad line : %s" %line)
 
@@ -132,20 +143,45 @@ def update_attributes(dest, src, dry_run=False, delete=False) :
 
             if not dry_run:
                 dest.setncattr(key, val)
+def cmp_var(var,  vardef:Variable) :
+    return var.dtype == vardef.type and var.dimensions == tuple(vardef.dimensions)
+
+def create_or_replace_var(ncfile, vardef:Variable, dry_run=False) :
+
+    if vardef.name in ncfile.variables:
+
+        var = ncfile.variables[vardef.name]
+
+        if cmp_var(var, vardef):
+            # Same var -> skipping
+            return
+
+        # Different vars
+        if dry_run:
+            info("Would replace var : %s" % vardef.name)
+        else:
+            info("Replacing var : %s" % vardef.name)
+            del ncfile.variables[vardef.name]
+
+    if dry_run :
+        info("Would add variable : %s" % vardef.name)
+        return
+
+    least_significant_digit = vardef.attributes.get("least_significant_digit", None)
+    fill_value = vardef.attributes.get("_FillValue", None)
+
+    info("Adding variable '%s'. Precision:%s" %  (vardef.name, least_significant_digit))
+
+    ncfile.createVariable(
+        vardef.name, vardef.type, vardef.dimensions,
+        zlib=True,
+        complevel=9,
+        least_significant_digit=least_significant_digit,
+        fill_value=fill_value)
 
 def initVar(ncfile, vardef:Variable, dry_run=False, delete_attrs=False) :
 
-    # Already there, skipping
-    if not vardef.name in ncfile.variables and not dry_run:
-        info("Adding variable '%s'", vardef.name)
-
-        least_significant_digit = vardef.attributes.get("least_significant_digit", None)
-
-        ncfile.createVariable(
-            vardef.name, vardef.type, vardef.dimensions,
-            zlib=True,
-            complevel=9,
-            least_significant_digit=least_significant_digit)
+    create_or_replace_var(ncfile, vardef, dry_run)
 
     var = ncfile.variables[vardef.name]
 
@@ -171,25 +207,43 @@ def cdl2netcdf(ncfile, cdl: CDL, dry_run=False, delete_attrs=False) :
 
 def init_nc(netcdf, properties, data_vars=DATA_VARS, dry_run=False, delete_attrs=False) :
 
-    cdl = parse_cdl(read_res(CDL_PATH), properties)
+    try:
+        # Try to load custom CDL first
+        custom_name = properties["Network_ID"] + ".cdl"
+        cdl = parse_cdl(read_res(custom_name), properties)
+        info("Used custom CDL : %s" % custom_name)
+
+    except FileNotFoundError:
+        cdl = parse_cdl(read_res(CDL_PATH), properties)
+
+    # Ensures all requested data vars are defined
+    missing_vars = set(data_var for data_var in data_vars if data_var not in cdl.variables)
+    if len(missing_vars) > 0 :
+        raise Exception("Unknown data vars : %s" % missing_vars)
 
     filtered_cdl = deepcopy(cdl)
 
     # Filter data vars (variables with "time" dimension)
     # Also adds the "Time" variable
-    filtered_cdl.variables = dict((key, var)
-                         for key, var in cdl.variables.items()
-                         if (not "time" in var.dimensions) or var.name in data_vars + [TIME_VAR])
+    filtered_cdl.variables = dict()
+    for key, var in cdl.variables.items() :
+        if "time" in var.dimensions and not key in data_vars + [TIME_VAR] :
+            continue
+        filtered_cdl.variables[key] = var
+
+
+
 
     cdl2netcdf(netcdf, filtered_cdl, dry_run, delete_attrs)
 
     if not dry_run :
+
         # Init scalar vars
         netcdf.variables[LONGITUDE_VAR][0] = properties["Station_Longitude"]
         netcdf.variables[LATITUDE_VAR][0] = properties["Station_Latitude"]
         netcdf.variables[ELEVATION_VAR][0] = properties["Station_Elevation"]
 
-        fillShortName(netcdf, properties["Station_ID"])
+        fill_str(netcdf, STATION_NAME_VAR, properties["Station_ID"])
 
     return cdl
 
